pub struct TraceRow<'a, Expr> {
pub binary_poly: &'a [Expr],
pub arbitrary_poly: &'a [Expr],
pub int: &'a [Expr],
}Expand description
A view on a row of the trace. Contains references to cells of the trace of all types lying in the same trace row.
Fields
binary_poly: &'a [Expr]arbitrary_poly: &'a [Expr]int: &'a [Expr]Implementations
Sourceimpl<'a, Expr> TraceRow<'a, Expr>
impl<'a, Expr> TraceRow<'a, Expr>
Sourcepub fn from_slice_with_layout(row: &'a [Expr], layout: &ColumnLayout) -> Self
pub fn from_slice_with_layout(row: &'a [Expr], layout: &ColumnLayout) -> Self
Given a slice that represents a raw row of the trace,
creates a TraceRow from it.
Subdivides the slice according to the given column layout.