pub struct UairTrace<'a, PolyCoeff: Clone, Int: Clone, const D: usize> {
pub binary_poly: Cow<'a, [DenseMultilinearExtension<BinaryPoly<D>>]>,
pub arbitrary_poly: Cow<'a, [DenseMultilinearExtension<DensePolynomial<PolyCoeff, D>>]>,
pub int: Cow<'a, [DenseMultilinearExtension<Int>]>,
}Expand description
The trace of a UAIR execution (pre-projection). If owned, it contains the full trace, otherwise it contains a view on the full trace (e.g. only public columns).
Fields
binary_poly: Cow<'a, [DenseMultilinearExtension<BinaryPoly<D>>]>arbitrary_poly: Cow<'a, [DenseMultilinearExtension<DensePolynomial<PolyCoeff, D>>]>int: Cow<'a, [DenseMultilinearExtension<Int>]>Implementations
Sourceimpl<PolyCoeff: Clone, Int: Clone, const D: usize> UairTrace<'static, PolyCoeff, Int, D>
impl<PolyCoeff: Clone, Int: Clone, const D: usize> UairTrace<'static, PolyCoeff, Int, D>
Sourcepub fn public(&self, sig: &UairSignature) -> UairTrace<'_, PolyCoeff, Int, D>
pub fn public(&self, sig: &UairSignature) -> UairTrace<'_, PolyCoeff, Int, D>
Returns a sub-trace containing only public columns. Returned trace is borrowed from the full trace.
Sourcepub fn witness(&self, sig: &UairSignature) -> UairTrace<'_, PolyCoeff, Int, D>
pub fn witness(&self, sig: &UairSignature) -> UairTrace<'_, PolyCoeff, Int, D>
Returns a sub-trace containing only witness columns. Returned trace is borrowed from the full trace.